@@ -1,12 +1,24 @@
 # Author: Ingo Feinerer
 
-# The "..." are additional arguments for the FunctionGenerator parser
-setGeneric("TextDocCol", function(object, parser = readPlain, load = FALSE, ...) standardGeneric("TextDocCol"))
+# The "..." are additional arguments for the FunctionGenerator reader
+setGeneric("TextDocCol", function(object,
+                                  readerControl = list(reader = object@DefaultReader, language = "en_US", load = FALSE),
+                                  dbControl = list(useDb = FALSE, dbName = "", dbType = "DB1"),
+                                  ...) standardGeneric("TextDocCol"))
 setMethod("TextDocCol",
           signature(object = "Source"),
-          function(object, parser = readPlain, load = FALSE, ...) {
-              if (inherits(parser, "FunctionGenerator"))
-                  parser <- parser(...)
+          function(object,
+                   readerControl = list(reader = object@DefaultReader, language = "en_US", load = FALSE),
+                   dbControl = list(useDb = FALSE, dbName = "", dbType = "DB1"),
+                   ...) {
+              if (attr(readerControl$reader, "FunctionGenerator"))
+                  readerControl$reader <- readerControl$reader(...)
+
+              if (dbControl$useDb) {
+                  if (!dbCreate(dbControl$dbName, dbControl$dbType))
+                      stop("error in creating database")
+                  db <- dbInit(dbControl$dbName, dbControl$dbType)
+              }
 
               tdl <- list()
               counter <- 1
@@ -16,18 +28,32 @@ setMethod("TextDocCol",
                   # If there is no Load on Demand support
                   # we need to load the corpus into memory at startup
                   if (!object@LoDSupport)
-                      load <- TRUE
-                  tdl <- c(tdl, list(parser(elem, load, as.character(counter))))
+                      readerControl$load <- TRUE
+                  doc <- readerControl$reader(elem, readerControl$load, readerControl$language, as.character(counter))
+                  if (dbControl$useDb) {
+                      dbInsert(db, ID(doc), doc)
+                      tdl <- c(tdl, ID(doc))
+                  }
+                  else
+                      tdl <- c(tdl, list(doc))
                   counter <- counter + 1
               }
 
-              dmeta.df <- data.frame(MetaID = rep(0, length(tdl)), stringsAsFactors = FALSE)
+              df <- data.frame(MetaID = rep(0, length(tdl)), stringsAsFactors = FALSE)
+              if (dbControl$useDb) {
+                  dbInsert(db, "DMetaData", df)
+                  dmeta.df <- data.frame(key = "DMetaData", subset = I(list(NA)))
+                  dbDisconnect(db)
+              }
+              else
+                  dmeta.df <- df
+
               cmeta.node <- new("MetaDataNode",
                             NodeID = 0,
                             MetaData = list(create_date = Sys.time(), creator = Sys.getenv("LOGNAME")),
                             children = list())
 
-              return(new("TextDocCol", .Data = tdl, DMetaData = dmeta.df, CMetaData = cmeta.node))
+              return(new("TextDocCol", .Data = tdl, DMetaData = dmeta.df, CMetaData = cmeta.node, DBControl = dbControl))
           })
 
 setGeneric("loadDoc", function(object, ...) standardGeneric("loadDoc"))
@@ -69,7 +95,7 @@ setMethod("loadDoc",
                   mail <- readLines(con)
                   close(con)
                   Cached(object) <- TRUE
-                  for (index in seq(along = mail)) {
+                  for (index in seq_along(mail)) {
                       if (mail[index] == "")
                           break
                   }
@@ -80,14 +106,19 @@ setMethod("loadDoc",
               }
           })
 
-setGeneric("tmUpdate", function(object, origin, parser = readPlain, ...) standardGeneric("tmUpdate"))
+setGeneric("tmUpdate", function(object,
+                                origin,
+                                readerControl = list(reader = origin@DefaultReader, language = "en_US", load = FALSE),
+                                ...) standardGeneric("tmUpdate"))
 # Update is only supported for directories
 # At the moment no other LoD devices are available anyway
 setMethod("tmUpdate",
           signature(object = "TextDocCol", origin = "DirSource"),
-          function(object, origin, parser = readPlain, load = FALSE, ...) {
-              if (inherits(parser, "FunctionGenerator"))
-                  parser <- parser(...)
+          function(object, origin,
+                   readerControl = list(reader = origin@DefaultReader, language = "en_US", load = FALSE),
+                   ...) {
+              if (inherits(readerControl$reader, "FunctionGenerator"))
+                  readerControl$reader <- readerControl$reader(...)
 
               object.filelist <- unlist(lapply(object, function(x) {as.character(URI(x))[2]}))
               new.files <- setdiff(origin@FileList, object.filelist)
@@ -95,7 +126,7 @@ setMethod("tmUpdate",
               for (filename in new.files) {
                   elem <- list(content = readLines(filename),
                                uri = substitute(file(filename)))
-                  object <- appendElem(object, parser(elem, load, filename))
+                  object <- appendElem(object, readerControl$reader(elem, readerControl$load, readerControl$language, filename))
               }
 
               return(object)
@@ -107,7 +138,17 @@ setMethod("tmMap",
           function(object, FUN, ...) {
               result <- object
               # Note that text corpora are automatically loaded into memory via \code{[[}
-              result@.Data <- lapply(object, FUN, ..., DMetaData = DMetaData(object))
+              if (DBControl(object)[["useDb"]]) {
+                  db <- dbInit(DBControl(object)[["dbName"]], DBControl(object)[["dbType"]])
+                  i <- 1
+                  for (id in unlist(object)) {
+                      db[[id]] <- FUN(object[[i]], ..., DMetaData = DMetaData(object))
+                      i <- i + 1
+                  }
+                  dbDisconnect(db)
+              }
+              else
+                  result@.Data <- lapply(object, FUN, ..., DMetaData = DMetaData(object))
               return(result)
           })
 
@@ -128,6 +169,13 @@ setMethod("asPlain",
 
               return(FUN(xmlRoot(corpus), ...))
           })
+setMethod("asPlain",
+          signature(object = "NewsgroupDocument"),
+          function(object, FUN, ...) {
+              new("PlainTextDocument", .Data = Corpus(object), Cached = TRUE, URI = "", Author = Author(object),
+                  DateTimeStamp = DateTimeStamp(object), Description = Description(object), ID = ID(object),
+                  Origin = Origin(object), Heading = Heading(object), Language = Language(object))
+          })
 
 setGeneric("tmTolower", function(object, ...) standardGeneric("tmTolower"))
 setMethod("tmTolower",
@@ -145,22 +193,31 @@ setMethod("stripWhitespace",
               return(object)
           })
 
-setGeneric("stemDoc", function(object, ...) standardGeneric("stemDoc"))
+setGeneric("stemDoc", function(object, language = "english", ...) standardGeneric("stemDoc"))
 setMethod("stemDoc",
           signature(object = "PlainTextDocument"),
-          function(object, ...) {
-              require("Rstem")
+          function(object, language = "english", ...) {
               splittedCorpus <- unlist(strsplit(object, " ", fixed = TRUE))
-              stemmedCorpus <- Rstem::wordStem(splittedCorpus)
+              stemmedCorpus <- if (require("Rstem"))
+                  Rstem::wordStem(splittedCorpus, language)
+              else
+                  SnowballStemmer(splittedCorpus, Weka_control(S = language))
               Corpus(object) <- paste(stemmedCorpus, collapse = " ")
               return(object)
           })
 
+setGeneric("removePunctuation", function(object, ...) standardGeneric("removePunctuation"))
+setMethod("removePunctuation",
+          signature(object = "PlainTextDocument"),
+          function(object, ...) {
+              Corpus(object) <- gsub("[[:punct:]]+", "", Corpus(object))
+              return(object)
+          })
+
 setGeneric("removeWords", function(object, stopwords, ...) standardGeneric("removeWords"))
 setMethod("removeWords",
           signature(object = "PlainTextDocument", stopwords = "character"),
           function(object, stopwords, ...) {
-              require("Rstem")
               splittedCorpus <- unlist(strsplit(object, " ", fixed = TRUE))
               noStopwordsCorpus <- splittedCorpus[!splittedCorpus %in% stopwords]
               Corpus(object) <- paste(noStopwordsCorpus, collapse = " ")
@@ -188,57 +245,24 @@ setMethod("tmIndex",
           })
 
 sFilter <- function(object, s, ...) {
-    query.df <- DMetaData(object)
     con <- textConnection(s)
     tokens <- scan(con, "character")
     close(con)
-    local.meta <- lapply(object, LocalMetaData)
-    local.used.meta <- lapply(local.meta, function(x) names(x) %in% tokens)
-    l.meta <- NULL
-    for (i in 1:length(object)) {
-        l.meta <- c(l.meta, list(local.meta[[i]][local.used.meta[[i]]]))
-    }
-    # Load local meta data from text documents into data frame
-    for (i in 1:length(l.meta)) {
-        l.meta[[i]] <- c(l.meta[[i]], list(author = Author(object[[i]])))
-        l.meta[[i]] <- c(l.meta[[i]], list(datetimestamp = DateTimeStamp(object[[i]])))
-        l.meta[[i]] <- c(l.meta[[i]], list(description = Description(object[[i]])))
-        l.meta[[i]] <- c(l.meta[[i]], list(identifier = ID(object[[i]])))
-        l.meta[[i]] <- c(l.meta[[i]], list(origin = Origin(object[[i]])))
-        l.meta[[i]] <- c(l.meta[[i]], list(heading = Heading(object[[i]])))
-    }
-    for (i in 1:length(l.meta)) {
-        for (j in 1:length(l.meta[[i]])) {
-            m <- l.meta[[i]][[j]]
-            m.name <- names(l.meta[[i]][j])
-            if (!(m.name %in% names(query.df))) {
-                before <- rep(NA, i - 1)
-                after <- rep(NA, length(l.meta) - i)
-                if (length(m) > 1) {
-                    nl <- vector("list", length(l.meta))
-                    nl[1:(i-1)] <- before
-                    nl[i] <- list(m)
-                    nl[(i+1):length(l.meta)] <- after
-                    insert <- data.frame(I(nl), stringsAsFactors = FALSE)
-                }
-                else
-                    insert <- c(before, m, after)
-                query.df <- cbind(query.df, insert, stringsAsFactors = FALSE)
-                names(query.df)[length(query.df)] <- m.name
-            }
-            else {
-                if (is.null(m))
-                    m <- NA
-                if (length(m) > 1) {
-                    rl <- query.df[ , m.name]
-                    rl[i] <- list(m)
-                    query.df[ , m.name] <- data.frame(I(rl), stringsAsFactors = FALSE)
-                }
-                else
-                    query.df[i, m.name] <- m
-            }
-        }
-    }
+    localMetaNames <- unique(names(sapply(object, LocalMetaData)))
+    localMetaTokens <- localMetaNames[localMetaNames %in% tokens]
+    n <- names(DMetaData(object))
+    tags <- c("Author", "DateTimeStamp", "Description", "ID", "Origin", "Heading", "Language", localMetaTokens)
+    query.df <- DMetaData(prescindMeta(object, tags))
+    if (DBControl(object)[["useDb"]])
+        DMetaData(object) <- DMetaData(object)[, setdiff(n, tags), drop = FALSE]
+    # Rename to avoid name conflicts
+    names(query.df)[names(query.df) == "Author"] <- "author"
+    names(query.df)[names(query.df) == "DateTimeStamp"] <- "datetimestamp"
+    names(query.df)[names(query.df) == "Description"] <- "description"
+    names(query.df)[names(query.df) == "ID"] <- "identifier"
+    names(query.df)[names(query.df) == "Origin"] <- "origin"
+    names(query.df)[names(query.df) == "Heading"] <- "heading"
+    names(query.df)[names(query.df) == "Language"] <- "language"
     attach(query.df)
     try(result <- rownames(query.df) %in% row.names(query.df[eval(parse(text = s)), ]))
     detach(query.df)
@@ -256,8 +280,17 @@ setGeneric("appendElem", function(object, data, meta = NULL) standardGeneric("ap
 setMethod("appendElem",
           signature(object = "TextDocCol", data = "TextDocument"),
           function(object, data, meta = NULL) {
-              object@.Data[[length(object)+1]] <- data
-              object@DMetaData <- rbind(object@DMetaData, c(MetaID = CMetaData(object)@NodeID, meta))
+              if (DBControl(object)[["useDb"]]) {
+                  db <- dbInit(DBControl(object)[["dbName"]], DBControl(object)[["dbType"]])
+                  if (dbExists(db, ID(data)))
+                      warning("document with identical ID already exists")
+                  dbInsert(db, ID(data), data)
+                  dbDisconnect(db)
+                  object@.Data[[length(object)+1]] <- ID(data)
+              }
+              else
+                  object@.Data[[length(object)+1]] <- data
+              DMetaData(object) <- rbind(DMetaData(object), c(MetaID = CMetaData(object)@NodeID, meta))
               return(object)
           })
 
@@ -265,9 +298,10 @@ setGeneric("appendMeta", function(object, cmeta = NULL, dmeta = NULL) standardGe
 setMethod("appendMeta",
           signature(object = "TextDocCol"),
           function(object, cmeta = NULL, dmeta = NULL) {
-              object@CMetaData@MetaData <- c(object@CMetaData@MetaData, cmeta)
-              if (!is.null(cmeta))
-                  object@DMetaData <- cbind(object@DMetaData, dmeta)
+              object@CMetaData@MetaData <- c(CMetaData(object)@MetaData, cmeta)
+              if (!is.null(dmeta)) {
+                  DMetaData(object) <- cbind(DMetaData(object), eval(dmeta))
+              }
               return(object)
           })
 
@@ -275,12 +309,10 @@ setGeneric("removeMeta", function(object, cname = NULL, dname = NULL) standardGe
 setMethod("removeMeta",
           signature(object = "TextDocCol"),
           function(object, cname = NULL, dname = NULL) {
-              if (!is.null(cname)) {
+              if (!is.null(cname))
                   object@CMetaData@MetaData <- CMetaData(object)@MetaData[names(CMetaData(object)@MetaData) != cname]
-              }
-              if (!is.null(dname)) {
-                  object@DMetaData <- DMetaData(object)[names(DMetaData(object)) != dname]
-              }
+              if (!is.null(dname))
+                  DMetaData(object) <- DMetaData(object)[, names(DMetaData(object)) != dname, drop = FALSE]
               return(object)
           })
 
@@ -289,12 +321,12 @@ setMethod("prescindMeta",
           signature(object = "TextDocCol", meta = "character"),
           function(object, meta) {
               for (m in meta) {
-                  if (m %in% c("Author", "DateTimeStamp", "Description", "ID", "Origin", "Heading")) {
+                  if (m %in% c("Author", "DateTimeStamp", "Description", "ID", "Origin", "Heading", "Language")) {
                       local.m <- lapply(object, m)
                       local.m <- lapply(local.m, function(x) if (is.null(x)) return(NA) else return(x))
                       local.m <- unlist(local.m)
-                      object@DMetaData <- cbind(DMetaData(object), data.frame(m = local.m), stringsAsFactors = FALSE)
-                      names(object@DMetaData)[length(object@DMetaData)] <- m
+                      DMetaData(object) <- cbind(DMetaData(object), data.frame(m = local.m, stringsAsFactors = FALSE))
+                      names(DMetaData(object))[which(names(DMetaData(object)) == "m")] <- m
                   }
                   else {
                       local.meta <- lapply(object, LocalMetaData)
@@ -304,8 +336,8 @@ setMethod("prescindMeta",
                           local.m <- unlist(local.m)
                       else
                           local.m <- I(local.m)
-                      object@DMetaData <- cbind(DMetaData(object), data.frame(m = local.m), stringsAsFactors = FALSE)
-                      names(object@DMetaData)[length(object@DMetaData)] <- m
+                      DMetaData(object) <- cbind(DMetaData(object), data.frame(m = local.m, stringsAsFactors = FALSE))
+                      names(DMetaData(object))[which(names(DMetaData(object)) == "m")] <- m
                   }
               }
               return(object)
@@ -319,9 +351,15 @@ setMethod("[",
 
               object <- x
               object@.Data <- x@.Data[i, ..., drop = FALSE]
-              df <- as.data.frame(DMetaData(object)[i, ])
-              names(df) <- names(DMetaData(object))
-              object@DMetaData <- df
+              if (DBControl(object)[["useDb"]]) {
+                  index <- object@DMetaData[[1 , "subset"]]
+                  if (any(is.na(index)))
+                      object@DMetaData[[1 , "subset"]] <- i
+                  else
+                      object@DMetaData[[1 , "subset"]] <- index[i]
+              }
+              else
+                  DMetaData(object) <- DMetaData(x)[i, , drop = FALSE]
               return(object)
           })
 
@@ -329,21 +367,49 @@ setMethod("[<-",
           signature(x = "TextDocCol", i = "ANY", j = "ANY", value = "ANY"),
           function(x, i, j, ... , value) {
               object <- x
-              object@.Data[i, ...] <- value
+              if (DBControl(object)[["useDb"]]) {
+                  db <- dbInit(DBControl(object)[["dbName"]], DBControl(object)[["dbType"]])
+                  counter <- 1
+                  for (id in object@.Data[i, ...]) {
+                      if (length(value) == 1)
+                          db[[id]] <- value
+                      else {
+                          db[[id]] <- value[[counter]]
+                      }
+                      counter <- counter + 1
+                  }
+                  dbDisconnect(db)
+              }
+              else
+                  object@.Data[i, ...] <- value
               return(object)
           })
 
 setMethod("[[",
           signature(x = "TextDocCol", i = "ANY", j = "ANY"),
           function(x, i, j, ...) {
-              return(loadDoc(x@.Data[[i]]))
+              if (DBControl(x)[["useDb"]]) {
+                  db <- dbInit(DBControl(x)[["dbName"]], DBControl(x)[["dbType"]])
+                  result <- dbFetch(db, x@.Data[[i]])
+                  dbDisconnect(db)
+                  return(loadDoc(result))
+              }
+              else
+                  return(loadDoc(x@.Data[[i]]))
           })
 
 setMethod("[[<-",
           signature(x = "TextDocCol", i = "ANY", j = "ANY", value = "ANY"),
           function(x, i, j, ..., value) {
               object <- x
-              object@.Data[[i, ...]] <- value
+              if (DBControl(object)[["useDb"]]) {
+                  db <- dbInit(DBControl(object)[["dbName"]], DBControl(object)[["dbType"]])
+                  index <- object@.Data[[i]]
+                  db[[index]] <- value
+                  dbDisconnect(db)
+              }
+              else
+                  object@.Data[[i, ...]] <- value
               return(object)
           })
 
@@ -379,13 +445,16 @@ setMethod("c",
           signature(x = "TextDocCol"),
           function(x, ..., meta = list(merge_date = Sys.time(), merger = Sys.getenv("LOGNAME")), recursive = TRUE) {
               args <- list(...)
-              if(length(args) == 0)
+              if (length(args) == 0)
                   return(x)
 
+              if (!all(sapply(args, inherits, "TextDocCol")))
+                  stop("not all arguments are text document collections")
+              if (DBControl(x)[["useDb"]] == TRUE || any(unlist(sapply(args, DBControl)["useDb", ])))
+                  stop("concatenating text document collections with activated database is not supported")
+
               result <- x
               for (c in args) {
-                  if (!inherits(c, "TextDocCol"))
-                      stop("invalid argument")
                   result <- c2(result, c)
               }
               return(result)
@@ -399,6 +468,9 @@ setMethod("c2",
               # Concatenate data slots
               object@.Data <- c(as(x, "list"), as(y, "list"))
 
+              # Set the DBControl slot
+              object@DBControl <- list(useDb = FALSE, dbName = "", dbType = "DB1")
+
               # Update the CMetaData tree
               cmeta <- new("MetaDataNode", NodeID = 0, MetaData = meta, children = list(CMetaData(x), CMetaData(y)))
               update.struct <- update_id(cmeta)
@@ -444,7 +516,6 @@ setMethod("c2",
               return(object)
           })
 
-
 setMethod("c",
           signature(x = "TextDocument"),
           function(x, ..., recursive = TRUE){
@@ -458,7 +529,11 @@ setMethod("c",
                             MetaData = list(create_date = Sys.time(), creator = Sys.getenv("LOGNAME")),
                             children = list())
 
-              return(new("TextDocCol", .Data = list(x, ...), DMetaData = dmeta.df, CMetaData = cmeta.node))
+              return(new("TextDocCol",
+                         .Data = list(x, ...),
+                         DMetaData = dmeta.df,
+                         CMetaData = cmeta.node,
+                         DBControl = list(useDb = FALSE, dbName = "", dbType = "DB1")))
           })
 
 setMethod("length",
@@ -486,9 +561,9 @@ setMethod("summary",
                                               "\nThe metadata consists of %d tag-value pairs and a data frame\n"),
                                        length(CMetaData(object)@MetaData)))
                   cat("Available tags are:\n")
-                  cat(names(CMetaData(object)@MetaData), "\n")
+                  cat(strwrap(paste(names(CMetaData(object)@MetaData), collapse = " "), indent = 2, exdent = 2), "\n")
                   cat("Available variables in the data frame are:\n")
-                  cat(names(DMetaData(object)), "\n")
+                  cat(strwrap(paste(names(DMetaData(object)), collapse = " "), indent = 2, exdent = 2), "\n")
               }
     })
 
@@ -498,7 +573,13 @@ setMethod("inspect",
           function(object) {
               summary(object)
               cat("\n")
-              show(object@.Data)
+              if (DBControl(object)[["useDb"]]) {
+                  db <- dbInit(DBControl(object)[["dbName"]], DBControl(object)[["dbType"]])
+                  show(dbMultiFetch(db, unlist(object)))
+                  dbDisconnect(db)
+              }
+              else
+                  show(object@.Data)
           })
 
 # No metadata is checked
@@ -506,5 +587,38 @@ setGeneric("%IN%", function(x, y) standardGeneric("%IN%"))
 setMethod("%IN%",
           signature(x = "TextDocument", y = "TextDocCol"),
           function(x, y) {
-              x %in% y
+              if (DBControl(y)[["useDb"]]) {
+                  db <- dbInit(DBControl(y)[["dbName"]], DBControl(y)[["dbType"]])
+                  result <- any(sapply(y, function(x, z) {x %in% Corpus(z)}, x))
+                  dbDisconnect(db)
+              }
+              else
+                  result <- x %in% y
+              return(result)
+          })
+
+setMethod("lapply",
+          signature(X = "TextDocCol"),
+          function(X, FUN, ...) {
+              if (DBControl(X)[["useDb"]]) {
+                  db <- dbInit(DBControl(X)[["dbName"]], DBControl(X)[["dbType"]])
+                  result <- lapply(dbMultiFetch(db, unlist(X)), FUN, ...)
+                  dbDisconnect(db)
+              }
+              else
+                  result <- base::lapply(X, FUN, ...)
+              return(result)
+          })
+
+setMethod("sapply",
+          signature(X = "TextDocCol"),
+          function(X, FUN, ..., simplify = TRUE, USE.NAMES = TRUE) {
+              if (DBControl(X)[["useDb"]]) {
+                  db <- dbInit(DBControl(X)[["dbName"]], DBControl(X)[["dbType"]])
+                  result <- sapply(dbMultiFetch(db, unlist(X)), FUN, ...)
+                  dbDisconnect(db)
+              }
+              else
+                  result <- base::sapply(X, FUN, ...)
+              return(result)
           })
